Hey,

Thin version <%= Thin::VERSION::STRING %> (codename <%= Thin::VERSION::CODENAME %>) is out!

== What's new?

<%= changelog %>

== Get it!
 
Install Thin from RubyGems:

 gem install thin

== Contribute

Site:            http://code.macournoyer.com/thin/
Group:           http://groups.google.com/group/thin-ruby/topics
Issues:          https://github.com/macournoyer/thin/issues
Security issues: macournoyer+thinsecurity@gmail.com
Code:            http://github.com/macournoyer/thin
IRC:             #thin on freenode

Thanks to all the people who contributed to Thin, EventMachine, Rack and Mongrel.

Marc-Andre Cournoyer
http://macournoyer.com/